<?php

class Cryptophoto_CryptophotoAdmin_Plugin extends Atmail_Controller_Plugin
{

  protected $_pluginFullName = 'Cryptophoto Administrator Settings';
  protected $_pluginAuthor = 'Cryptophoto <Tech@CryptoPhoto.com>';
  protected $_pluginDescription = 'Improves security by adding the Cryptophoto two-factor authentication';
  protected $_pluginCopyright = '2013 Cryptophoto';
  protected $_pluginUrl = '';
  protected $_pluginNotes = '';
  protected $_pluginVersion = '1.20131017';
  protected $_pluginCompat = '6.1.5';
  protected $_pluginModule = 'admin';
  private $_loginPage = false;

  // class constructor
  public function __construct()
  {
    $this->_pluginDescription = "Admin Management for Cryptophoto";
    $this->log = Zend_Registry::get('log');
    parent::__construct();
  }


  /*
  * creates the Cryptophoto tables upon installation
  */
  public function setup()
  {
    if(!$this->dbValid('#__cryptophotokeys')) {
      $db = Zend_Registry::get("dbAdapter");
      $db->query("CREATE TABLE `#__cryptophotokeys` (`rowid` int(11) unsigned NOT NULL auto_increment, `publickey` varchar(35) NOT NULL DEFAULT '', `privatekey` varchar(35) NOT NULL DEFAULT '', `salt` varchar(35) NOT NULL DEFAULT '', PRIMARY KEY  (`rowid`))");
      $db->query("INSERT INTO `#__cryptophotokeys` (`publickey`,`privatekey`,`salt`) VALUES('','','')");
    }

    if(!$this->dbValid('#__cryptophoto')) {
      $db->query("CREATE TABLE `#__cryptophoto` (`rowid` int(11) unsigned NOT NULL auto_increment, `account` varchar(128) NOT NULL DEFAULT '', `enabledCP` varchar(11) NOT NULL DEFAULT '',`mailserver` varchar(64) NOT NULL DEFAULT '',`password` varchar(128) NOT NULL DEFAULT '',`expire` varchar(15) DEFAULT NULL, PRIMARY KEY  (`rowid`))");
    }
  }
  
  
  /*
  * adds the Cryptophoto menu item in the settings section of the admin account
  */
  public function renderSettingsTabNav()
  {
    $this->log->debug("**\n Admin: renderMailSettingsNav \n**\n");

    $params = array(
      "title" => "CryptoPhoto Settings",
      "text"  => "CryptoPhoto",
      "icon"  => "images/cryptophoto.png"
    );
    
    echo $this->_createMailSettingsNavSection($params);
  }


  /*
  * The Cryptophoto settings section of the admin account
  */
  public function settings()
  {
    $this->_initView();

    $this->log->debug("**\n Admin: settings \n**\n");
    $db = Zend_Registry::get("dbAdapter");

    if($this->dbValid('#__cryptophotokeys')) {
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];

      //check for existing Cryptophoto keys
      if($settings != null) {
        $this->view->cpvalid = true;
        $this->view->cp_publickey  = $settings['publickey'] ? $settings['publickey'] : "";
        $this->view->cp_privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
        $this->view->cp_salt       = $settings['salt'] ? $settings['salt'] : "";
      }
      // Cryptophoto keys not found or missing table
      else {
        $this->view->value = false;
        $this->view->err_message = "Could not detect a valid Cryptophoto installation";
        return;
      }
    }
    else {
      $this->view->value = false;
      $this->view->err_message = "Could not detect a valid Cryptophoto setup - Cryptophoto keys missing or invalid installation";
      return;
    } 

    // render the admin settings page
    echo $this->view->render("settings.phtml");

  }


  /*
  * saves the site's Cryptophoto configuration when changing the keys and salt  
  */
  public function saveSettings()
  {
    $this->_initView();

    require_once("library/jQuery/jQuery.php");

    if($this->dbValid('#__cryptophotokeys')) {
      // fetch the current Cryptophoto settings
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];
      $publickey  = $settings['publickey'] ? $settings['publickey'] : "";
      $privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
      $salt       = $settings['salt'] ? $settings['salt'] : "";
      $rowid      = $settings['rowid'];

      // non-empty salt 
      if (!empty($_POST['cp_salt'])) {
        $salt  = $_POST['cp_salt'];
      }

      // non-empty privatekey
      if (!empty($_POST['cp_privatekey'])) {
        $privatekey  = $_POST['cp_privatekey'];
      }

      // non-empty publickey
      if (!empty($_POST['cp_publickey'])) {
        $publickey  = $_POST['cp_publickey'];
      }

      $db->query("UPDATE `#__cryptophotokeys` SET `publickey`=?, `privatekey`=?, `salt`=? WHERE `rowid`=?", array(trim($publickey),trim($privatekey),$salt,$rowid));


      jQuery::addMessage("The settings have been updated");
    }
    else {    
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
      return;
    }

    echo $this->view->render("jsonresponse.phtml", null, true);
  }

  /*
  * Tests the configuration of the plugin
  */

  public function testSetup()
  {
    $this->_initView();
    require_once("library/jQuery/jQuery.php");

    if($this->dbValid('#__cryptophotokeys')) {
      // fetch the current Cryptophoto settings
      $db = Zend_Registry::get("dbAdapter");

      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];
      $publickey  = $settings['publickey'] ? $settings['publickey'] : "";
      $privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
      $salt       = $settings['salt'] ? $settings['salt'] : "";

      if($publickey && $privatekey && $salt) {
        require_once(dirname(__FILE__) . "/CryptoPhotoUtils.php");
        $cp = new CryptoPhotoUtils($privatekey, $publickey, md5($salt));
        $answer = $cp->start_session($_SERVER['REMOTE_ADDR']);

        if($answer[0] == TRUE) {
 
          jQuery::addMessage("Plugin is setup properly");

        } else {

          if ($answer[1] == "incorrect-timestamp") {
            jQuery::addError('Error: Please verify that the date/time is properly set on this server');
          } 
  
          else if (strpos($answer[1], "ip-not-allowed") !== false) {
            jQuery::addError('Error: The IP of this server(' . $answer[2] . ') is not added to the list of allowed IPs. Check your settings in your CryptoPhoto.com account');
          } 
  
          else {
            jQuery::addError('Error: Please verify that the API keys are valid');
          }
        }
      } else {
        jQuery::addError('Error: Please configure/save your API Keys and salt before testing the setup');
      }

    } else {
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
      return;
    }

    echo $this->view->render("jsonresponse.phtml", null, true);

  }


  /*
  * Displays the users list and their Cryptophoto status
  */
  public function userslist()
  {
    $this->_initView();
    $request = $this->getRequest()->getParams();
    $db = Zend_Registry::get("dbAdapter");

    if($this->dbValid('#__cryptophoto')) {

      // check for a valid post (non-empty CP status and user id)
      if(!empty($request['enable']) && !empty($request['id'])) {
        // enable Cryptophoto for a certain user
        $sql = "UPDATE `#__cryptophoto` SET enabledCP = 1 WHERE ";
        $where = str_repeat ( "rowid = ? OR ", count($request['id']) );
        $where = preg_replace('/OR $/', '', $where);
        $sql = $sql . $where;          
        $db->query($sql, $request['id']);
      } else if (!empty($request['disable']) && !empty($request['id'])) {
        // disable Cryptophoto for a certain user
        $sql = "UPDATE `#__cryptophoto` SET enabledCP = 0 WHERE ";
        $where = str_repeat ( "rowid = ? OR ", count($request['id']) );
        $where = preg_replace('/OR $/', '', $where);
        $sql = $sql . $where;
        $db->query($sql, $request['id']);
      }

      // search filter by username
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto`");
      $filter = !empty($request['filter']) ? $request['filter'] : '';

      // check for posted filter
      if ($filter) {
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` where account like ?", array("%".$filter."%"));
      } else {
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto`");
      }

      // check for posted page number; default page number = 1
      $page = !empty($request['page']) ? $request['page'] : 1;
      $paginator = Zend_Paginator::factory($db_values);

      // pagination settings
      $paginator->setItemCountPerPage(10);
      $paginator->setCurrentPageNumber($page);
      $this->view->userslist = $paginator;
    }
    else {
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
    }

    if( !isset($this->view->jsonIdsToRender) )
      $this->view->jsonIdsToRender = array();

    $this->view->jsonIdsToRender['users-table'] = 'listusers.phtml';

    echo $this->view->render('jsonresponse.phtml', null, true);
  }


  /*
  * Checks that the Cryptophoto table exists
  */
  public function dbValid($tableName) {

    $db = Zend_Registry::get("dbAdapter");
    $db_values = $db->fetchAll("SHOW TABLES LIKE '" . $tableName . "'");   

    if(count($db_values)) {
      return true;
    }
    else {
      return false;
    }
  }

}
